import os
import json


class BaseScript:

    def __init__(self, name, runner, params):
        self.name = name
        self.runner = runner
        self.params = params
        self.script_params = params['scripts'][name]

    def save_to_json(self, data, output_dir):
        with open(os.path.join(output_dir, self.name + '.json'), 'w') as f:
            json.dump(data, f, indent=4)

    def execute(self, output_dir):
        raise NotImplementedError()



import os
import json
import argparse
import importlib
import datetime

from barbell2.utils import Logger, current_time_secs, elapsed_secs


class ScriptRunner:

    def __init__(self, params):
        self.params = params
        self.scripts = []
        os.makedirs(params['log_dir'], exist_ok=True)
        self.logger = Logger(prefix='log_etl', to_dir=params['log_dir'])

    def get_output_dir(self):
        os.makedirs(self.params['output_dir'], exist_ok=True)
        now = datetime.datetime.now()
        timestamp = '{}'.format(now.strftime('%Y%m%d%H%M%S'))
        os.makedirs(os.path.join(self.params['output_dir'], timestamp), exist_ok=True)
        output_dir = os.path.join(self.params['output_dir'], timestamp)
        return output_dir

    def execute(self):
        if len(self.scripts) == 0:
            self.logger.print('Nothing to execute')
            return
        output_dir = self.get_output_dir()
        start_overall = current_time_secs()
        for script in self.scripts:
            self.logger.print('Starting script {}'.format(script.name))
            start = current_time_secs()
            script.execute(output_dir)
            self.logger.print(
                'Script finished after {} seconds'.format(elapsed_secs(start)))
        self.logger.print(
            'Elapsed overall: {} seconds'.format(elapsed_secs(start_overall)))
        os.system('touch {}'.format(os.path.join(output_dir, 'finished.txt')))


HELP = """
Path to parameter file (default: params.json)

Example parameter file:

{
    "scripts": {
        "DummyScript": {},
        "RetrieveStudyListScript": {},
        "RetrieveProcedureCountsAndComplicationsPerQuarterScript": {
            "study_name": "ESPRESSO_v2.0_DPCA",
            "surgery_date_field_name": "dpca_datok",
            "complications_field_name": "dpca_compl"
        }
    },
    "output_dir": "./castordashboard_data",
    "use_cache": false,
    "verbose": true,
    "websocket_origin": "137.120.191.233:5006",
    "port_nr": 5006
}

The "scripts" item contains a dictionary of script (class) names as keys and various script-specific
settings as values.  For example, the RetrieveProcedureCountsAndComplicationsPerQuarterScript script
requires a study name to access the right study in Castor EDC.

The other parameter settings specify where to store the script output JSON data. 
"""


def main():

    parser = argparse.ArgumentParser()
    parser.add_argument('--params', help=HELP, default='params.json')
    args = parser.parse_args()
    with open(args.params, 'r') as f:
        params = json.load(f)

    runner = ScriptRunner(params)
    for script_name in params['scripts'].keys():
        script = getattr(importlib.import_module('castordashboard.etl.scripts'), script_name)
        runner.scripts.append(script(script_name, runner, params))
    runner.execute()


if __name__ == '__main__':
    main()



from . import BaseScript


class DummyScript(BaseScript):

    def __init__(self, name, runner, params):
        super(DummyScript, self).__init__(name, runner, params)

    def execute(self, output_dir):
        self.save_to_json(self.params, output_dir)



from barbell2.castorclient import CastorClient
from . import BaseScript


class RetrieveStudyListScript(BaseScript):

    def __init__(self, name, runner, params):
        super(RetrieveStudyListScript, self).__init__(name, runner, params)

    def execute(self, output_dir):
        client = CastorClient()
        study_list = client.get_studies()
        data = []
        for study in study_list:
            data.append(str(study))
        self.save_to_json(data, output_dir)



from .basescript import BaseScript
from .dummyscript import DummyScript
from .retrievestudylistscript import RetrieveStudyListScript
from .retrieveprocedurecountsandcomplicationsperquarterscript import \
    RetrieveProcedureCountsAndComplicationsPerQuarterScript
